@@ -20,7 +20,6 @@ import static java.util.Optional.ofNullable;
 
 import java.io.IOException;
 import java.io.InputStream;
-import java.net.URL;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
@@ -103,13 +102,12 @@ final class FontMapperImpl implements FontMapper
 
         try
         {
-            String ttfName = "org/sejda/sambox/resources/ttf/LiberationSans-Regular.ttf";
-            URL url = FontMapper.class.getClassLoader().getResource(ttfName);
-            if (url == null)
+            String ttfName = "/org/sejda/sambox/resources/ttf/LiberationSans-Regular.ttf";
+            InputStream ttfStream = FontMapper.class.getResourceAsStream(ttfName);
+            if (ttfStream == null)
             {
                 throw new IOException("Error loading resource: " + ttfName);
             }
-            InputStream ttfStream = url.openStream();
             TTFParser ttfParser = new TTFParser();
             lastResortFont = ttfParser.parse(ttfStream);
         }
