@@ -540,6 +540,16 @@ public class PDExtendedGraphicsState implements COSObjectable
         return BlendMode.getInstance(dict.getDictionaryObject(COSName.BM));
     }
 
+    /**
+     * Set the blending mode.
+     * 
+     * @param bm
+     */
+    public void setBlendMode(BlendMode bm)
+    {
+        dict.setItem(COSName.BM, BlendMode.getCOSName(bm));
+    }
+
     /**
      * Returns the soft mask stored in the COS dictionaryO
      *
@@ -547,6 +557,10 @@ public class PDExtendedGraphicsState implements COSObjectable
      */
     public PDSoftMask getSoftMask()
     {
+        if (!dict.containsKey(COSName.SMASK))
+        {
+            return null;
+        }
         return PDSoftMask.create(dict.getDictionaryObject(COSName.SMASK));
     }
 
