@@ -57,7 +57,7 @@ import org.slf4j.LoggerFactory;
  * @author Stephan Gerhard
  * @author Ben Litchfield
  */
-class AppearanceGeneratorHelper
+public class AppearanceGeneratorHelper
 {
     private static final Logger LOG = LoggerFactory.getLogger(AppearanceGeneratorHelper.class);
 
@@ -101,7 +101,7 @@ class AppearanceGeneratorHelper
      * @param field the field which you wish to control the appearance of
      * @throws IOException
      */
-    AppearanceGeneratorHelper(PDVariableText field) throws IOException
+    public AppearanceGeneratorHelper(PDVariableText field) throws IOException
     {
         this.field = field;
         validateAndEnsureAcroFormResources();
@@ -446,6 +446,18 @@ class AppearanceGeneratorHelper
                 .orElseGet(() -> defaultAppearance.getFont());
 
         requireNotNullArg(font, "font is null, check whether /DA entry is incomplete or incorrect");
+        if (font.getName().contains("+"))
+        {
+            LOG.warn("Font '" + defaultAppearance.getFontName().getName() + "' of field '"
+                    + field.getFullyQualifiedName() + "' contains subsetted font '" + font.getName()
+                    + "'");
+            LOG.warn("This may bring trouble with PDField.setValue(), PDAcroForm.flatten() or "
+                    + "PDAcroForm.refreshAppearances()");
+            LOG.warn("You should replace this font with a non-subsetted font:");
+            LOG.warn("PDFont font = PDType0Font.load(doc, new FileInputStream(fontfile), false);");
+            LOG.warn("acroForm.getDefaultResources().put(COSName.getPDFName(\""
+                    + defaultAppearance.getFontName().getName() + "\", font);");
+        }
 
         float fontSize = defaultAppearance.getFontSize();
 
